/*
  (c) 2002 Bertrik Sikken, bertrik@zonnet.nl
  
  Transport layer for communication with HP5400/5470 scanner.
  
*/  
  
#include <stdio.h>        /* fprintf */    
#include <usb.h>          /* libusb include file */

#include "hp5400_xfer.h"


#define DBG fprintf
#define DBG_MSG stdout
#define DBG_ERR stderr

#define LIBUSB_TIMEOUT 10000


int hp5400_open(void)
{
  struct usb_bus *bus;
  struct usb_device *dev;
  usb_dev_handle *devLibUsb;

  usb_init();
  usb_find_busses();
  usb_find_devices();

  for (bus = usb_busses; bus; bus = bus->next) {
    for (dev = bus->devices; dev; dev = dev->next) {
      DBG(DBG_MSG, "Bus %s Dev %s : 0x%04X-0x%04X\n", bus->dirname, dev->filename,
          dev->descriptor.idVendor, dev->descriptor.idProduct);

      if (dev->descriptor.idVendor == 0x3F0) {
        if ((dev->descriptor.idProduct == 0x1005) ||
            (dev->descriptor.idProduct == 0x1105)) {

          DBG(DBG_MSG, "Found HP5400 or HP5470\n");
          devLibUsb = usb_open(dev);
          if (usb_claim_interface(devLibUsb, 0) < 0) {
            DBG(DBG_ERR, "ERROR: usb_claim_interface failed\n");
            usb_close(devLibUsb);
            continue;
          }
          /* everything OK, return the handle */
          return (int)devLibUsb;
        }
      }
    }
  }
  /* in case nothing was found */
  return -1;
}


void hp5400_close(int iHandle)
{
  usb_dev_handle *LibUsbHandle;

  if (iHandle < 0) {
    return;
  }
  LibUsbHandle = (usb_dev_handle *)iHandle;

  usb_release_interface(LibUsbHandle, 0);
  usb_close(LibUsbHandle);
}


/* returns value > 0 if verify ok */
int hp5400_command_verify(int iHandle, int iCmd)
{
  usb_dev_handle *LibUsbHandle;
  unsigned char abData[2];

  if (iHandle < 0) {
    DBG(DBG_ERR, "hp5400_command_verify: invalid handle\n");
    return -1;
  }
  LibUsbHandle = (usb_dev_handle *)iHandle;

  /* command 0xc500: read back previous command */
  usb_control_msg(LibUsbHandle, 0xC0, 0x04, 0xc500, 0, (char *)abData,
                  2, LIBUSB_TIMEOUT);
  
  if (abData[0] != (iCmd >> 8)) {
    DBG(DBG_ERR, "hp5400_command_verify failed, expected %d, got %d",
  (int)(iCmd >> 8), (int)abData[0]);
    return -1;
  }
  return 1;
}


/* returns > 0 if command OK */
int hp5400_command_read(int iHandle, int iCmd, int iLen, unsigned char *pbData)
{
  usb_dev_handle *LibUsbHandle;
  int iRequest;

  if (iHandle < 0) {
    DBG(DBG_ERR, "hp5400_command_read: invalid handle\n");
    return -1;
  }
  LibUsbHandle = (usb_dev_handle *)iHandle;

  iRequest = (iLen > 1) ? 0x04 : 0x0C;
  usb_control_msg(LibUsbHandle, 0xC0, iRequest, iCmd, 0, (char *)pbData,
                  iLen, LIBUSB_TIMEOUT);

  return hp5400_command_verify(iHandle, iCmd);
}


/* returns >0 if command OK */
int hp5400_command_write(int iHandle, int iCmd, int iLen, unsigned char *pbData)
{
  usb_dev_handle *LibUsbHandle;
  int iRequest;

  if (iHandle < 0) {
    DBG(DBG_ERR, "hp5400_command_write: invalid handle\n");
    return -1;
  }
  LibUsbHandle = (usb_dev_handle *)iHandle;

  iRequest = (iLen > 1) ? 0x04 : 0x0C;
  usb_control_msg(LibUsbHandle, 0x40, iRequest, iCmd, 0, (char *)pbData,
                  iLen, LIBUSB_TIMEOUT);

  return hp5400_command_verify(iHandle, iCmd);
}


